﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

namespace Collision
{
    public partial class MainForm : Form
    {
        /// <summary>
        /// Spiel-Dauer eines Tick-Abstandes
        /// (interne Spiel-Geschwindigkeit)
        /// </summary>
        private const float Clock = 5;

        private enum Crashes
        {
            /// <summary>beide rasen aufeinander zu, selbe Achse</summary>
            DirectCrash180,
            /// <summary>einer steht, selbe Achse</summary>
            DirectCrash,
            /// <summary>beide rasen aufeinander zu, verschobene Achse</summary>
            ParallelCrash180,
            /// <summary>einer steht, verschobene Achse</summary>
            ParallelCrash,
            /// <summary>beide rasen senkrecht aufeinander zu, verschobene Achse</summary>
            OrthogonalCrash90,
        }

        private List<Sphaere> Spheres = new List<Sphaere>();
        private List<Wall> Walls = new List<Wall>();
        private object LockObj = new object();
        private bool Inject = false;
        private Sphaere NewSphaere = null;

        public MainForm()
        {
            this.InitializeComponent();

            this.Walls.Add(new Wall(Wall.KindWall.North));
            this.Walls.Add(new Wall(Wall.KindWall.Ost));
            this.Walls.Add(new Wall(Wall.KindWall.South));
            this.Walls.Add(new Wall(Wall.KindWall.West));

            this.cbSpecialCases.DataSource = Enum.GetNames(typeof(Crashes));
            this.cbSpecialCases.SelectedIndex = 3;
        }

        private void pictureBox1_Paint(object sender, PaintEventArgs e)
        {
            for (int i = 0; i < this.Spheres.Count; i++)
            {
                this.Spheres[i].Draw(e.Graphics);
            }
        }

        private void btnNext_Click(object sender, EventArgs e)
        {
            lock (this.LockObj)
            {
                if (!this.Inject)
                {
                    // etwas Platz um die neue Kugel
                    this.NewSphaere = new Sphaere(20);
                    this.Inject = true;
                }
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            lock (this.LockObj)
            {
                this.InjectNew();
                this.CheckWall();
                this.CheckCrash();
                this.pictureBox1.Invalidate();
            }
        }

        /// <summary>
        /// die nächste Kugel einfügen,
        /// wenn genügend Platz ist
        /// </summary>
        private void InjectNew()
        {
            if (!this.Inject)
            {
                return;
            }

            for (int i = 0; i < this.Spheres.Count; i++)
            {
                if (Sphaere.CheckCrash(this.Spheres[i], this.NewSphaere))
                {
                    return;
                }
            }

            // original Durchmesser restaurieren,
            // zur Überprüfung des Platzes wurde ein größerer verwendet.
            this.NewSphaere.Radius = 10;
            this.Spheres.Add(this.NewSphaere);
            this.NewSphaere = null;
            this.Inject = false;
        }

        private void CheckCrash()
        {
            for (int i = 0; i < this.Spheres.Count; i++)
            {
                for (int j = i + 1; j < this.Spheres.Count; j++)
                {
                    if (Sphaere.CheckCrash(this.Spheres[i], this.Spheres[j]))
                    {
                        this.Spheres[i].CheckWall(this.pictureBox1.ClientRectangle);
                        this.Spheres[j].CheckWall(this.pictureBox1.ClientRectangle);
                    }
                }
            }
        }

        private void CheckWall()
        {
            for (int i = 0; i < this.Spheres.Count; i++)
            {
                this.Spheres[i].Step(MainForm.Clock);
                this.Spheres[i].CheckWall(this.pictureBox1.ClientRectangle);
            }
        }

        private void cbTimer_CheckedChanged(object sender, EventArgs e)
        {
            this.timer1.Enabled = this.cbTimer.Checked;
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            lock (this.LockObj)
            {
                this.Spheres.Clear();
                this.pictureBox1.Invalidate();
            }
        }

        private void btnDoIt_Click(object sender, EventArgs e)
        {
            // damit dann die Anzeige stimmt
            this.cbTimer.Checked = false;
            lock (this.LockObj)
            {
                this.Spheres.Clear();
                Sphaere.ResetColor();
            }

            Rectangle rc = this.pictureBox1.ClientRectangle;
            float rad1 = 10;
            float rad2 = 10;
            float mas1 = 1;
            float mas2 = 1;

            if (this.rbMass1.Checked)
            {
                mas1 = 2;
                rad1 = 14;
            }
            else if (this.rbMass2.Checked)
            {
                mas2 = 2;
                rad2 = 14;
            }

            float y1 = rc.Height / 2F;
            float x1 = 0 + 2 * rad1;
            float x2 = rc.Width - 2 * rad2;
            float difference = (float)this.nudDifference.Value;
            Vector v1 = null;
            Vector v2 = null;
            Vector l1 = null;
            Vector l2 = null;
            Crashes special = (Crashes)this.cbSpecialCases.SelectedIndex;
            switch (special)
            {
                case Crashes.DirectCrash:
                case Crashes.DirectCrash180:
                    v1 = new Vector(1, 0, 0);
                    v2 = new Vector(-1, 0, 0);
                    l1 = new Vector(x1, y1, 0);
                    l2 = new Vector(x2, y1, 0);
                    break;
                case Crashes.ParallelCrash:
                case Crashes.ParallelCrash180:
                    v1 = new Vector(1, 0, 0);
                    v2 = new Vector(-1, 0, 0);
                    l1 = new Vector(x1, y1 - rad1 * difference, 0);
                    l2 = new Vector(x2, y1, 0);
                    break;
                case Crashes.OrthogonalCrash90:
                    v1 = new Vector(1, 0, 0);
                    v2 = new Vector(0, 1, 0);
                    l1 = new Vector(x1, y1 - rad1 * difference, 0);
                    l2 = new Vector(y1, x1, 0);
                    break;
                default:
                    return;
            }

            switch (special)
            {
                case Crashes.DirectCrash:
                case Crashes.ParallelCrash:
                    // Objekt 2 steht still in der Mitte des Feldes
                    v2 = new Vector();
                    l2 = new Vector(rc.Width / 2F, y1, 0);
                    break;
            }

            Sphaere s1 = new Sphaere(rad1, mas1, l1, v1);
            Sphaere s2 = new Sphaere(rad2, mas2, l2, v2);
            this.Spheres.Add(s1);
            this.Spheres.Add(s2);

            // damit die Anzeige stimmt
            this.cbTimer.Checked = true;
        }

        /// <summary>
        /// Entfernen der angeklickten Kugel,
        /// wenn der Timer ausgeschaltet ist
        /// </summary>
        private void pictureBox1_DoubleClick(object sender, EventArgs e)
        {
            if (this.timer1.Enabled)
            {
                return;
            }

            Sphaere del = new Sphaere(this.pictureBox1.PointToClient(Cursor.Position));
            for (int i = 0; i < this.Spheres.Count; i++)
            {
                del.Radius = this.Spheres[i].Radius;
                if (Sphaere.CheckCrash(this.Spheres[i], del))
                {
                    this.Spheres.RemoveAt(i);
                    this.pictureBox1.Invalidate();
                    return;
                }
            }
        }
    }
}
