﻿using System;

namespace Collision
{
    /// <summary>
    /// Klasse zur Beschreibung eines Vektors,
    /// sowohl Ort als auch Geschwindigkeit.
    /// </summary>
    class Vector
    {
        public float X;
        public float Y;
        public float Z;

        public Vector(float x, float y, float z)
        {
            this.X = x;
            this.Y = y;
            this.Z = z;
        }

        public Vector(Vector other)
        {
            this.X = other.X;
            this.Y = other.Y;
            this.Z = other.Z;
        }

        public Vector()
        {
            this.X = 0;
            this.Y = 0;
            this.Z = 0;
        }

        internal Vector Clone()
        {
            return new Vector(this);
        }

        /// <summary>
        /// Berechnung der Position von this
        /// mit der Geschwindigkeit zum Zeitpunkt
        /// </summary>
        /// <param name="speed"></param>
        /// <param name="duration"></param>
        public void ChangePosition(Vector speed, float duration)
        {
            this.X += duration * speed.X;
            this.Y += duration * speed.Y;
            this.Z += duration * speed.Z;
        }

        /// <summary>
        /// unäre Subtraktion
        /// </summary>
        /// <param name="v1"></param>
        /// <param name="v2"></param>
        /// <returns></returns>
        public static Vector operator -(Vector v1, Vector v2)
        {
            return new Vector(v1.X - v2.X, v1.Y - v2.Y, v1.Z - v2.Z);
        }

        /// <summary>
        /// Berechnung der Länge des Vektors
        /// </summary>
        /// <returns>die Länge</returns>
        public float Length()
        {
            return (float)(Math.Sqrt(this.X * this.X + this.Y * this.Y + this.Z * this.Z));
        }

        /// <summary>
        /// unäre Addition
        /// </summary>
        /// <param name="v1"></param>
        /// <param name="v2"></param>
        /// <returns></returns>
        public static Vector operator +(Vector v1, Vector v2)
        {
            return new Vector(v1.X + v2.X, v1.Y + v2.Y, v1.Z + v2.Z);
        }

        /// <summary>
        /// Vektor multipliziert mit Skalar
        /// </summary>
        /// <param name="d"></param>
        /// <param name="v"></param>
        /// <returns></returns>
        public static Vector operator *(float d, Vector v)
        {
            return new Vector(d * v.X, d * v.Y, d * v.Z);
        }

        /// <summary>
        /// Skalarprodukt
        /// </summary>
        /// <param name="v1"></param>
        /// <param name="v2"></param>
        /// <returns></returns>
        public static float operator *(Vector v1, Vector v2)
        {
            return v1.X * v2.X + v1.Y * v2.Y + v1.Z * v2.Z;
        }

        /// <summary>
        /// Vektor dividiert durch Skalar
        /// </summary>
        /// <param name="v"></param>
        /// <param name="d"></param>
        /// <returns></returns>
        public static Vector operator /(Vector v, float d)
        {
            return new Vector(v.X / d, v.Y / d, v.Z / d);
        }

        public void Normalize()
        {
            float d = this.Length();
            if (d == 0)
            {
                this.X = 1;
                return;
            }

            this.X /= d;
            this.Y /= d;
            this.Z /= d;
        }

        public static Vector Kreuzprodukt(Vector v1, Vector v2)
        {
            float x = v1.Y * v2.Z - v1.Z * v2.Y;
            float y = v1.Z * v2.X - v1.X * v2.Z;
            float z = v1.X * v2.Y - v1.Y * v2.X;
            return new Vector(x, y, z);
        }

        /// <summary>
        /// zur Anzeige im Debugger
        /// </summary>
        public override string ToString()
        {
            return string.Format("{{X={0}, Y={1}, Z={2}}}", this.X, this.Y, this.Z);
        }
    }
}
